//
// Rille.cpp
// A linear feature -- rille, fault, or valley -- on the moon.
//
//    Copyright 2002 by Akkana Peck.
// You may use or distribute this code under the terms of the GPL.
//

#include "Rille.h"

#include <qpainter.h>
#include <math.h>
#include <string.h>
#include <malloc.h>
#include <stdlib.h>
#include <stdio.h>

// Static variables:
Rille::Rille() : MoonFeature()
{
}

Rille::Rille(char* name, char* desc, int size, int angle,
             int longitude, int latitude)
{
    Init(name, desc, size, angle, longitude, latitude);
}

void
Rille::Init(char* name, char* desc, int size, int angle,
             int longitude, int latitude)
{
    MoonFeature::Init(name, desc, size, longitude, latitude);
    mAngle = angle;
}

Rille::~Rille()
{
}

int Rille::GetSizeX()
{
    return (int)((double)mSize * cos(mAngle * M_PI / 180.));
}

int Rille::GetSizeY()
{
    return (int)((double)mSize * sin(mAngle * M_PI / 180.));
}

void Rille::PaintFeature(QPainter* painter, int cX, int cY, int radius,
                         int mirrored, int librationE, int librationN)
{
    double longRadians = M_PI/180.* (GetLongitude() + librationE);
    double latRadians = M_PI/180. * (GetLatitude() + librationN);
    cX += mirrored * (int)(radius * sin(longRadians));
    cY += (int)(-radius * sin(latRadians));
    double length = mSize * radius / 1609.;
    int dx = -(int)(length * cos(M_PI/180.*mAngle) * mirrored);
    int dy = (int)(length * sin(M_PI/180.*mAngle));

    painter->setPen(QColor(50, 255, 200));
    painter->drawLine(cX - dx, cY - dy, cX + dx, cY + dy);
    //printf("Rille from (%d, %d) to (%d, %d)\n",
    //       cX - dx, cY - dy, cX + dx, cY + dy);

    LabelName(painter, cX - dx, cY - dy);
}

// Routines for looping over all known craters:
void
Rille::InitRilles()
{
    if (!sMoonFeatureList)
        return;

    char* rilleFileName = "/opt/Qtopia/share/qmoon/rimae.dat";
    FILE* cfile = fopen(rilleFileName, "r");
    if (!cfile)
        return;

    char line[BUFSIZ];
    while (fgets(line, sizeof line, cfile) != 0)
    {
        if (*line == '#')       // Skip comments
            continue;
        int latitude = (int)atof(line+30);
        if (line[36] == 'S')
            latitude = -latitude;
        int longitude = (int)atof(line+38);
        if (line[44] == 'W')
            longitude = -longitude;
        // Write a null after the name in the line buffer,
        // so we can pass it to strdup:
        int i;
        for (i=28; i>0 && line[i] == ' '; --i)
            ;
        if (i <= 0)
            return;

        line[i+1] = '\0';

        sMoonFeatureList[sNumFeatures++] = new Rille(line, line+61,
                                                     (int)atof(line+47),
                                                     (int)atof(line+55),
                                                     longitude, latitude);
    }
    printf("Rilles loaded, %d features\n", sNumFeatures);
}


